//////////////////////////////////////////////
// main.cpp
//
//////////////////////////////////////////////

/// Includes ---------------------------------

// nkLog
// REGION_CPP nkLogInclude
#include <NilkinsLog/ConsoleLogger.h>
// !REGION_CPP nkLogInclude

// nkGraphics
// REGION_CPP nkGraphicsLogInclude
#include <NilkinsGraphics/Log/LogManager.h>
// !REGION_CPP nkGraphicsLogInclude
// REGION_CPP nkGraphicsContextInclude
#include <NilkinsGraphics/RenderContexts/RenderContextManager.h>
// !REGION_CPP nkGraphicsContextInclude
// REGION_CPP nkGraphicsMainInclude
#include <NilkinsGraphics/Systems/MainSystem.h>
// !REGION_CPP nkGraphicsMainInclude

// Standards
// REGION_CPP standardMemoryInclude
#include <memory>
// !REGION_CPP standardMemoryInclude

/// Function ---------------------------------

int main ()
{
	// REGION_CPP nkLogSetup
	std::unique_ptr<nkLog::Logger> logger = std::make_unique<nkLog::ConsoleLogger>() ;
	nkGraphics::LogManager::getInstance()->setReceiver(logger.get()) ;
	// !REGION_CPP nkLogSetup

	// REGION_CPP nkGraphicsInit
	if (!nkGraphics::MainSystem::getInstance()->initialize())
		return -1 ;
	// !REGION_CPP nkGraphicsInit

	// REGION_CPP nkGraphicsContextCreation
	nkGraphics::RenderContext* context = nkGraphics::RenderContextManager::getInstance()->createRenderContextAndWindowFromConfig() ;
	// !REGION_CPP nkGraphicsContextCreation

	// REGION_CPP nkGraphicsSimpleRun
	nkGraphics::MainSystem::getInstance()->run(context) ;
	// !REGION_CPP nkGraphicsSimpleRun

	// REGION_CPP nkGraphicsClean
	nkGraphics::MainSystem::getInstance()->kill() ;
	// !REGION_CPP nkGraphicsClean
}