//////////////////////////////////////////////
// main.cpp
//
//////////////////////////////////////////////

/// Includes ---------------------------------

// nkLog
#include <NilkinsLog/ConsoleLogger.h>

// nkGraphics
// REGION_CPP nkGraphicsConfigInclude
#include <NilkinsGraphics/Configs/ConfigHolder.h>
#include <NilkinsGraphics/Configs/ConfigManager.h>

// Included because we will want to alter the window's size
#include <NilkinsGraphics/Configs/WindowDescriptor.h>
// !REGION_CPP nkGraphicsConfigInclude

#include <NilkinsGraphics/Log/LogManager.h>

#include <NilkinsGraphics/RenderContexts/RenderContextManager.h>

#include <NilkinsGraphics/Systems/MainSystem.h>

// nkResources
// REGION_CPP nkResourcesInclude
#include <NilkinsResources/ResourceManager.h>
// !REGION_CPP nkResourcesInclude

// Standards
#include <memory>

/// Function ---------------------------------

int main ()
{
	// Prepare logging
	std::unique_ptr<nkLog::Logger> logger = std::make_unique<nkLog::ConsoleLogger>() ;
	nkGraphics::LogManager::getInstance()->setReceiver(logger.get()) ;

	// REGION_CPP nkGraphicsConfigGet
	nkGraphics::ConfigHolder* config = nkGraphics::ConfigManager::getInstance()->getCurrentConfig() ;
	// !REGION_CPP nkGraphicsConfigGet

	// REGION_CPP nkGraphicsConfigSet
	config->_wantedApi = nkGraphics::RENDERING_API::D3D11 ;

	config->_window.setValue(nkGraphics::WINDOW_PROPS::WIDTH, 1280) ;
	config->_window.setValue(nkGraphics::WINDOW_PROPS::HEIGHT, 1024) ;
	// !REGION_CPP nkGraphicsConfigSet

	// This call will have no effect without the file
	// REGION_CPP nkGraphicsAutoConfig
	nkGraphics::MainSystem::getInstance()->autoConfigure() ;
	// !REGION_CPP nkGraphicsAutoConfig

	// REGION_CPP nkGraphicsResourcePathSet
	nkResources::ResourceManager::getInstance()->setWorkingPath("Data") ;
	// !REGION_CPP nkGraphicsResourcePathSet

	// Init the component
	if (!nkGraphics::MainSystem::getInstance()->initialize())
		return -1 ;

	// Prepare the context and window we will use
	nkGraphics::RenderContext* context = nkGraphics::RenderContextManager::getInstance()->createRenderContextAndWindowFromConfig() ;

	// Run and clean once done
	nkGraphics::MainSystem::getInstance()->run(context) ;
	nkGraphics::MainSystem::getInstance()->kill() ;
}