//////////////////////////////////////////////
// main.cpp
//
//////////////////////////////////////////////

/// Includes ---------------------------------

// nkLog
#include <NilkinsLog/ConsoleLogger.h>

// nkGraphics
#include <NilkinsGraphics/Log/LogManager.h>

// REGION_CPP nkGraphicsEntityInclude
#include <NilkinsGraphics/Graph/Entity.h>
#include <NilkinsGraphics/Graph/SubEntity.h>
// !REGION_CPP nkGraphicsEntityInclude

// REGION_CPP nkGraphicsNodeInclude
#include <NilkinsGraphics/Graph/Node.h>
#include <NilkinsGraphics/Graph/NodeManager.h>
// !REGION_CPP nkGraphicsNodeInclude

// REGION_CPP nkGraphicsMeshLoadInclude
#include <NilkinsGraphics/Meshes/Load/MeshLoader.h>
// !REGION_CPP nkGraphicsMeshLoadInclude

// REGION_CPP nkGraphicsMeshInclude
#include <NilkinsGraphics/Meshes/Mesh.h>
#include <NilkinsGraphics/Meshes/MeshManager.h>

// Needed to manipulate vertices from code
#include <NilkinsGraphics/Meshes/VertexComponent.h>
// !REGION_CPP nkGraphicsMeshInclude

#include <NilkinsGraphics/RenderContexts/RenderContextManager.h>

// REGION_CPP nkGraphicsRqInclude
#include <NilkinsGraphics/RenderQueues/RenderQueue.h>
#include <NilkinsGraphics/RenderQueues/RenderQueueManager.h>
// !REGION_CPP nkGraphicsRqInclude

#include <NilkinsGraphics/Systems/MainSystem.h>

// nkResources
#include <NilkinsResources/ResourceManager.h>

// Standards
#include <memory>

/// Internals --------------------------------

void prepareMeshFromCode ()
{
	// REGION_CPP nkGraphicsCreateMesh
	nkGraphics::Mesh* mesh = nkGraphics::MeshManager::getInstance()->createOrRetrieve("Triangle") ;
	// !REGION_CPP nkGraphicsCreateMesh

	// REGION_CPP nkGraphicsMeshMemoryRequest
	nkGraphics::VertexComponent* pointArray = mesh->getNewVertexArray(3) ;
	unsigned int* indexArray = mesh->getNewIndexArray(3) ;
	// !REGION_CPP nkGraphicsMeshMemoryRequest

	// REGION_CPP nkGraphicsMeshMemorySet
	// Vertex data
	pointArray[0]._position = nkMaths::Vector(-1.f, -1.f, 10.f) ;
	pointArray[1]._position = nkMaths::Vector(0.f, 1.f, 10.f) ;
	pointArray[2]._position = nkMaths::Vector(1.f, -1.f, 10.f) ;

	// Index data
	indexArray[0] = 0 ;
	indexArray[1] = 2 ;
	indexArray[2] = 1 ;
	// !REGION_CPP nkGraphicsMeshMemorySet

	// REGION_CPP nkGraphicsMeshComposition
	nkGraphics::VertexComposition compo ;

	compo._position = true ;
	compo._uv = false ;
	compo._normal = false ;

	compo._topology = nkGraphics::PRIMITIVE_TOPOLOGY_TRIANGLELIST ;

	mesh->setVertexComposition(compo) ;
	// !REGION_CPP nkGraphicsMeshComposition

	// REGION_CPP nkGraphicsMeshLoad
	mesh->load() ;
	// !REGION_CPP nkGraphicsMeshLoad
}

void prepareMeshFromFile ()
{
	nkGraphics::Mesh* mesh = nkGraphics::MeshManager::getInstance()->createOrRetrieve("Triangle") ;

	// REGION_CPP nkGraphicsMeshLoadFromFile
	nkGraphics::MeshLoader::fillBufferFromFile(mesh, "sphere.obj") ;
	mesh->load() ;
	// !REGION_CPP nkGraphicsMeshLoadFromFile
}

void addMeshToRenderQueue ()
{
	// REGION_CPP nkGraphicsRqGet
	nkGraphics::RenderQueue* rq = nkGraphics::RenderQueueManager::getInstance()->get(0) ;
	// !REGION_CPP nkGraphicsRqGet

	nkGraphics::Mesh* mesh = nkGraphics::MeshManager::getInstance()->createOrRetrieve("Triangle") ;

	// REGION_CPP nkGraphicsEntityLogic
	nkGraphics::Entity* ent = rq->addEntity() ;
	nkGraphics::SubEntity* subEnt = ent->addChild() ;

	subEnt->setMesh(mesh) ;
	// !REGION_CPP nkGraphicsEntityLogic
}

void addEntityToGraph ()
{
	nkGraphics::RenderQueue* rq = nkGraphics::RenderQueueManager::getInstance()->get(0) ;
	nkGraphics::Entity* ent = rq->getEntity(0) ;

	// REGION_CPP nkGraphicsNodeAssignAndMove
	nkGraphics::Node* node = nkGraphics::NodeManager::getInstance()->create() ;
	node->setPositionAbsolute(nkMaths::Vector(0.f, 0.f, 10.f)) ;

	ent->setParentNode(node) ;
	// !REGION_CPP nkGraphicsNodeAssignAndMove
}

/// Function ---------------------------------

int main ()
{
	// Prepare for logging
	std::unique_ptr<nkLog::Logger> logger = std::make_unique<nkLog::ConsoleLogger>() ;
	nkGraphics::LogManager::getInstance()->setReceiver(logger.get()) ;

	// For easiness
	nkResources::ResourceManager::getInstance()->setWorkingPath("Data") ;

	// Initialize and create context with window
	if (!nkGraphics::MainSystem::getInstance()->initialize())
		return -1 ;

	nkGraphics::RenderContext* context = nkGraphics::RenderContextManager::getInstance()->createRenderContextAndWindowFromConfig() ;

	// Prepare the mesh we will show
	//prepareMeshFromCode() ;
	prepareMeshFromFile() ;
	addMeshToRenderQueue() ;
	addEntityToGraph() ;

	// Run and clean
	nkGraphics::MainSystem::getInstance()->run(context) ;
	nkGraphics::MainSystem::getInstance()->kill() ;
}