﻿//////////////////////////////////////////////
// TargetOperations.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward declarations ---------------------

namespace nkGraphics
{
	class ClearTargetsPass ;
	class ComputePass ;
	class InstancingPass ;
	class MonoMaterialPass ;
	class Pass ;
	class PostProcessPass ;
	class RaytracingPass ;
	class RenderScenePass ;
	class Texture ;
	class Viewport ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkGraphics
#include <NilkinsExport/Exportable/Exportable.h>

// Standards
#include <string>
#include <utility>
#include <vector>

/// Class ------------------------------------

namespace nkGraphics
{	
	class DLL_GRAPHICS_EXPORT TargetOperations : public nkExport::Exportable
	{
		protected :
		
			// Attributs
			// Le nom pour le debug
			std::string _name ;

			// Les pass
			std::vector<Pass*> _passList ;

			// Les targets à prendre en compte
			std::vector<std::pair<Texture*, int>> _colorTargets ;
			Texture* _depthTarget ;

			// Target count
			unsigned int _targetsNumber ;

			// Savoir si nous sommes liés aux back buffers
			unsigned int _backBufferViewIndex ;
			bool _toBackBuffer ;
			bool _toChainDepthBuffer ;

			// Possibilite d'overload le Viewport pour rendus dans textures plus petites ou autres
			Viewport* _overloadedViewport ;
			Viewport* _tempViewport ;
		
		public :
		
			// Constructeur, destructeur
			TargetOperations () ;
			virtual ~TargetOperations () ;

			// Naming
			void setName (const char* name) ;
			const char* getName () const ;

			// Ajout de pass
			ClearTargetsPass* addClearTargetsPass (unsigned int index = -1) ;
			ComputePass* addComputePass (unsigned int index = -1) ;
			InstancingPass* addInstancingPass (unsigned int index = -1) ;
			MonoMaterialPass* addMonoMaterialPass (unsigned int index = -1) ;
			PostProcessPass* addPostProcessPass (unsigned int index = -1) ;
			RaytracingPass* addRaytracingPass (unsigned int index = -1) ;
			RenderScenePass* addRenderScenePass (unsigned int index = -1) ;
			// Obtention de pass
			unsigned int getPassCount () const ;
			Pass* getPass (unsigned int index) const ;
			// Manipulations
			virtual void deletePass (unsigned int index) ;
			virtual void changePassOrder (unsigned int fromIndex, unsigned int toIndex) ;

			// Concernant les targets
			virtual void setColorTarget (Texture* target, unsigned int index = 0, unsigned int targetViewId = 0) ;
			virtual void addColorTarget (Texture* target, unsigned int targetViewId = 0) ;
			virtual void changeColorTargetOrder (unsigned int fromIndex, unsigned int toIndex) ;
			virtual void removeColorTarget (unsigned int index) ;
			virtual void setDepthTarget (Texture* target) ;
			unsigned int getColorTargetCount () const ;
			Texture* getColorTarget (unsigned int index) const ;
			int getColorTargetRenderIndex (unsigned int index) const ;
			Texture* getDepthTarget () const ;
			virtual void setToBackBuffer (bool val, unsigned int indexInArray = 0) ;
			bool isToBackBuffer () const ;
			unsigned int getBackBufferViewIndex () const ;
			virtual void setToChainDepthBuffer (bool val) ;
			bool isToChainDepthBuffer () const ;

			// Viewport
			virtual void setOverloadedViewport (Viewport* value) ;
			Viewport* getOverloadedViewport () const ;

			// Exécution
			virtual void execute () = 0 ;

			// L'export et import
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
			// Pratique pour l'export
			void processColorTargetObjectNode (nkExport::Node* targetNode) ;
			void processPassObjectNode (nkExport::Node* passNode) ;
	} ;
}