﻿//////////////////////////////////////////////
// ProgramSourcesHolder.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Standards
#include <map>
#include <string>
#include <vector>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT ProgramSourcesHolder final
	{
		private :

			// Les mémoires
			// Ce seront des char* pour éviter les différences 
			std::string _vertexMemory ;
			std::string _hullMemory ;
			std::string _domainMemory ;
			std::string _geometryMemory ;
			std::string _pixelMemory ;
			std::string _computeMemory ;
			std::string _raytraceMemory ;

			std::vector<std::pair<std::string, std::string>> _macrosVertex ;
			std::vector<std::pair<std::string, std::string>> _macrosHull ;
			std::vector<std::pair<std::string, std::string>> _macrosDomain ;
			std::vector<std::pair<std::string, std::string>> _macrosGeometry ;
			std::vector<std::pair<std::string, std::string>> _macrosPixel ;
			std::vector<std::pair<std::string, std::string>> _macrosCompute ;
			std::vector<std::pair<std::string, std::string>> _macrosRaytrace ;

		public :

			// Constructeur, destructeur
			ProgramSourcesHolder () ;
			~ProgramSourcesHolder () ;

			// Getters
			std::string_view getVertexMemory () const ;
			std::string_view getHullMemory () const ;
			std::string_view getDomainMemory () const ;
			std::string_view getGeometryMemory () const ;
			std::string_view getPixelMemory () const ;
			std::string_view getComputeMemory () const ;
			std::string_view getRaytraceMemory () const ;
			std::pair<std::string_view, std::string_view> getDefineMacroVertex (unsigned int index) const ;
			std::pair<std::string_view, std::string_view> getDefineMacroHull (unsigned int index) const ;
			std::pair<std::string_view, std::string_view> getDefineMacroDomain (unsigned int index) const ;
			std::pair<std::string_view, std::string_view> getDefineMacroGeometry (unsigned int index) const ;
			std::pair<std::string_view, std::string_view> getDefineMacroPixel (unsigned int index) const ;
			std::pair<std::string_view, std::string_view> getDefineMacroCompute (unsigned int index) const ;
			std::pair<std::string_view, std::string_view> getDefineMacroRaytrace (unsigned int index) const ;
			std::vector<std::pair<std::string_view, std::string_view>> getDefineMacrosVertex () const ;
			std::vector<std::pair<std::string_view, std::string_view>> getDefineMacrosHull () const ;
			std::vector<std::pair<std::string_view, std::string_view>> getDefineMacrosDomain () const ;
			std::vector<std::pair<std::string_view, std::string_view>> getDefineMacrosGeometry () const ;
			std::vector<std::pair<std::string_view, std::string_view>> getDefineMacrosPixel () const ;
			std::vector<std::pair<std::string_view, std::string_view>> getDefineMacrosCompute () const ;
			std::vector<std::pair<std::string_view, std::string_view>> getDefineMacrosRaytrace () const ;
			unsigned int getDefineMacroMaxNumber () const ;
			unsigned int getDefineMacroMaxNumberPipeline () const ;
			unsigned int getDefineMacroNumberCompute () const ;
			unsigned int getDefineMacroNumberRaytrace () const ;

			// Setters
			void setVertexMemory (const std::string_view& str) ;
			void setHullMemory (const std::string_view& str) ;
			void setDomainMemory (const std::string_view& str) ;
			void setGeometryMemory (const std::string_view& str) ;
			void setPixelMemory (const std::string_view& str) ;
			void setComputeMemory (const std::string_view& str) ;
			void setRaytraceMemory (const std::string_view& str) ;
			void addDefineMacroVertex (const std::string_view& key, const std::string_view& value) ;
			void addDefineMacroHull (const std::string_view& key, const std::string_view& value) ;
			void addDefineMacroDomain (const std::string_view& key, const std::string_view& value) ;
			void addDefineMacroGeometry (const std::string_view& key, const std::string_view& value) ;
			void addDefineMacroPixel (const std::string_view& key, const std::string_view& value) ;
			void addDefineMacroCompute (const std::string_view& key, const std::string_view& value) ;
			void addDefineMacroRaytrace (const std::string_view& key, const std::string_view& value) ;

			// Utilitaires
			bool hasMinimalDefinitionsPipeline () const ;
			bool hasMinimalDefinitionsCompute () const ;
			bool hasMinimalDefinitionsGeometry () const ;
			bool hasMinimalDefinitionsRaytrace () const ;
	} ;
}