﻿//////////////////////////////////////////////
// Environment.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkScripts
{
	class EnvironmentStrategy ;
	class Function ;
	class Namespace ;
	class Script ;
	class ScriptObjectReference ;
	class UserType ;

	enum class INTERPRETER ;

	struct ExecutionResult ;
}

/// Includes ---------------------------------

// nkScripts
#include "../Dll/DllDefines.h"

#include "Functions/FunctionDefines.h"

// Standards
#include <string>
#include <unordered_map>
#include <vector>

/// Class ------------------------------------

namespace nkScripts
{
	class DLL_SCRIPTS_EXPORT Environment final
	{
		private :

			// Attributes
			// Name de l'environment
			std::string _name ;
			// List des fonctions actives
			std::unordered_map<std::string, Function*> _activeFunctions ;
			// Et des types actifs
			std::unordered_map<std::string, UserType*> _activeTypes ;
			// Namespaces actifs
			std::unordered_map<std::string, Namespace*> _activeNamespaces ;

			// L'environnement interne
			EnvironmentStrategy* _env ;

			// Interpreter Type actuel
			INTERPRETER _interpreterType ;

		public :

			// Constructeur, destructeur
			Environment () ;
			~Environment () ;

			// Getters
			const ExecutionResult& getLastExecutionResult () const ;
			const char* getName () const ;

			// Setters
			void setEnvironmentFor (INTERPRETER interpretType) ;
			void setName (const char* name) ;

			// Exécution
			bool execute (const Script& script) ;
			void requestGarbageCollection () ;

			// Manipulation mémoire environnement
			void setVar (const char* name, bool value) ;
			void setVar (const char* name, int value) ;
			void setVar (const char* name, float value) ;
			void setVar (const char* name, double value) ;
			void setVar (const char* name, const char* value) ;
			void setVar (const char* name, const ScriptObjectReference& value) ;
			void setObject (const char* name, const char* userTypeName, void* value) ;

			bool isVarSet (const char* name) ;

			bool getVar (const char* name, bool defaultValue) ;
			int getVar (const char* name, int defaultValue) ;
			float getVar (const char* name, float defaultValue) ;
			double getVar (const char* name, double defaultValue) ;
			const char* getVar (const char* name, const char* defaultValue) ;
			ScriptObjectReference getVar (const char* name) ;
			void* getObject (const char* name, const char* userTypeName = nullptr) ;
			ScriptObjectReference getScriptFunction (const char* name) ;

			// Ajout de fonctions
			Function* setFunc (const char* name) ;
			bool isFuncSet (const char* name) ;
			Function* getFunc (const char* name) ;

			// Ajout de types persos
			UserType* setUserType (const char* name) ;
			bool isUserTypeSet (const char* name) ;
			UserType* getUserType (const char* name) ;

			// Ajout de namespaces
			Namespace* setNamespace (const char* name) ;
			bool isNamespaceSet (const char* name) ;
			Namespace* getNamespace (const char* name) ;

			// Call C++ -> Script
			bool callScriptFunction (const ScriptObjectReference& reference, const DataStack& args, DataStack& expectedOutput) ;
			std::vector<unsigned char> serializeScriptObject (const ScriptObjectReference& reference, bool forwardUserDataOwnership = false) ;
			ScriptObjectReference deserializeScriptObject (const std::vector<unsigned char>& object) ;
	} ;
}

