﻿//////////////////////////////////////////////
// Component.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward declarations ---------------------

struct tagDRAWITEMSTRUCT ;
struct HWND__ ;

typedef HWND__* HWND ;
typedef tagDRAWITEMSTRUCT DRAWITEMSTRUCT ;

// Nous n'avons que du x64, ça devait le faire
typedef unsigned int UINT ;
typedef __int64 LONG_PTR ;
typedef unsigned __int64 UINT_PTR ;
typedef LONG_PTR LRESULT ;
typedef UINT_PTR WPARAM ;
typedef LONG_PTR LPARAM ;

typedef LRESULT (_stdcall* WNDPROC) (HWND, UINT, WPARAM, LPARAM) ;

namespace nkExport
{
	class Node ;
}

namespace nkWinUi
{
	enum class LAYOUT_TYPE ;

	class ContextMenu ;
	class ComponentGenerator ;
	class Layout ;
	class ScrollBar ;

	struct ChangeEventData ;
	struct KeyEventData ;
	struct WheelEventData ;
}

/// Includes ---------------------------------

// nkWinUi
#include "../Dll/DllDefines.h"

#include "../Inputs/HandledEventData.h"

#include "../Inputs/InputCodes.h"

#include "ComponentType.h"

// nkExport
#include <NilkinsExport/Exportable/Exportable.h>

// nkMaths
#include <NilkinsMaths/Geometry/Point.h>
#include <NilkinsMaths/Geometry/Rectangle.h>

// Standards
#include <string>
#include <vector>

/// Class ------------------------------------

namespace nkWinUi
{	
	class DLL_WINUI_EXPORT Component : public nkExport::Exportable
	{
		protected :
		
			// Attributs
			// Layout
			Layout* _layout ;

			// Tailles minimums et maximums
			std::pair<float, float> _minSize ;
			std::pair<float, float> _maxSize ;
			// Taille associée
			nkMaths::Rectangle _area ;

			// Ratio à respecter
			float _ratio ;
			// Le type de component
			COMPONENT_TYPE _componentType ;

			// Component parent
			Component* _parentComponent ;
			// Liste des fils pour pouvoir les update si besoin est
			std::vector<Component*> _children ;

			// Son nom aussi
			std::string _componentName ;
			// Label
			std::string _label ;

			// Un menu de contexte peut-être lié
			ContextMenu* _linkedContextMenu ;

			// Des données utilisateurs (pour les callbacks par exemple)
			void* _data ;

			// Ordre de depth dans l'interface
			unsigned int _zDepth ;

			// Si loadé
			bool _initialized ;
			// Enabled ou pas
			bool _enabled ;

		protected :

			// Utilitaires pour l'export
			virtual void _retrieveChildren (std::vector<Component*>& container) ;
		
		public :
		
			// Constructeur, destructeur
			Component () ;
			virtual ~Component () ;

			// Getters
			Layout* getLayout () const ;
			nkMaths::Point getPositionInParent () const ;
			nkMaths::Point getPositionInScreen () const ;
			float getWidth () const ;
			float getHeight () const ;
			nkMaths::Rectangle getArea () const ;
			virtual nkMaths::Point getBorderExtents () const ;
			virtual float getLayoutAllowedWidth () const ;
			virtual float getLayoutAllowedHeight () const ;
			virtual float getLayoutAllowedOffsetX () const ;
			virtual float getLayoutAllowedOffsetY () const ;
			bool isInitialized () const ;
			Component* getParentComponent () const ;
			COMPONENT_TYPE getComponentType () const ;
			const char* getComponentName () const ;
			bool getVisibility () const ;
			virtual const char* getLabel () const ;
			void* getData () const ;
			bool isEnabled () const ;
			bool isForegroundWindow () const ;
			unsigned int getZDepth () const ;

			// Setters
			virtual Layout* setLayout (LAYOUT_TYPE layoutType) ;
			virtual void setPositionInParent (nkMaths::Point position) ;
			virtual void setPositionInScreen (nkMaths::Point position) ;
			virtual void setSize (float width, float height) ;
			virtual void setWidth (float width) ;
			virtual void setHeight (float height) ;
			void setMinSize (float width, float height) ;
			void setMaxSize (float width, float height) ;
			void setRatio (float widthOverHeight) ;
			virtual void setArea (nkMaths::Rectangle area) ;
			virtual void setParentComponent (Component* parent, bool makeItVisible = true) ;
			virtual void setLabel (const std::string_view& label) ;
			void setComponentName (const std::string_view& name) ;
			virtual void setVisibility (bool value) ;
			void setData (void* data) ;
			void setEnabled (bool value) ;
				
			// Hierarchie
			void addChild (Component* child, bool makeItVisible = true) ;
			void removeChild (Component* child) ;
			void removeChild (unsigned int index) ;
			Component* getChild (unsigned int index) ;
			unsigned int getChildIndex (Component* child) ;
			void unloadWithChildren () ;
			void loadWithChildren () ;
			void updateZDepth () ;

			// Pratiques
			void bringToForeground () ;
			void focusWindow () ;

			// Conversions pratiques
			nkMaths::Point getCoordRelativeFromAbsolute (const nkMaths::Point& absCoords) ;
			nkMaths::Point getCoordAbsoluteFromRelative (const nkMaths::Point& relCoords) ;

			// Init
			virtual void load () = 0 ;
			virtual void unload () = 0 ;

			// Update
			virtual void updateWindow () ;

			// Events tenus par défaut
			void onSized () ;

			// Contexte menu si nécessaire lié
			ContextMenu* createContextMenu () ;
			ContextMenu* getCurrentContextMenu () ;
			void shutContextMenu () ;

			// Export / Import de base, à redéfinir par les fils s'il faut
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			// Celle-ci est appelée lors d'un export, une classe doit la réimplémenter pour l'export, plutôt que le exportClassToTree
			virtual void exportIntrospection (nkExport::Node* rootNode) ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}